from random import shuffle
import logging

from pyschieber.deck import Deck
from pyschieber.rules.stich_rules import stich_rules, card_allowed
from pyschieber.rules.count_rules import count_stich, counting_factor
from pyschieber.stich import PlayedCard

logger = logging.getLogger(__name__)


class Game:
    def __init__(self, teams=None, point_limit=2500, players=None):
        self.team_1 = teams[1]
        self.team_2 = teams[2]
        self.point_limit = point_limit
        self.players = players
        self.trumpf = None
        self.deck = Deck()
        self.stiche = []
        shuffle(self.deck.cards)

    def start(self):
        self.deal_cards()
        return self.play()

    def deal_cards(self):
        for i, card in enumerate(self.deck.cards):
            self.players[i % 4 + 1].set_card(card=card)

    def play(self):
        start_player_key = 1
        self.trumpf = self.players[start_player_key].choose_trumpf()
        logger.info('Chosen Trumpf: {0} \n'.format(self.trumpf))
        for i in range(9):
            stich = self.play_stich(start_player_key)
            self.count_points(stich, last=(i == 8))
            logger.info('\nStich: {0} \n'.format(stich.player))
            logger.info('{}{}\n'.format('-' * 180, self.trumpf))
            start_player_key = self.get_key(stich.player)
            self.stiche.append(stich)
            if self.point_limit <= self.team_1['points'] or self.point_limit <= self.team_2['points']:
                return True
        return False

    def get_key(self, player):
        for key, value in self.players.items():
            if player == value:
                return key

    def play_stich(self, start_player_key):
        first_card = self.play_card(first_card=None, player=self.players[start_player_key])
        played_cards = [PlayedCard(player=self.players[start_player_key], card=first_card)]
        for i in get_player_key(start_key=start_player_key):
            current_player = self.players[i]
            card = self.play_card(first_card=first_card, player=current_player)
            played_cards.append(PlayedCard(player=current_player, card=card))
        stich = stich_rules[self.trumpf](played_cards=played_cards)
        return stich

    def play_card(self, first_card, player):
        is_allowed_card = False
        generator = player.choose_card()
        chosen_card = next(generator)
        while not is_allowed_card:
            is_allowed_card = card_allowed(first_card=first_card, chosen_card=chosen_card, hand_cards=player.cards,
                                           trumpf=self.trumpf)
            card = generator.send(is_allowed_card)
            chosen_card = chosen_card if card is None else card
        else:
            logger.info('Table: {0}:{1}'.format(player, chosen_card))
            player.cards.remove(chosen_card)
        return chosen_card

    def count_points(self, stich, last):
        player = stich.player
        player_number = self.get_player_number(player)
        cards = [played_card.card for played_card in stich.played_cards]
        if player_number % 2 == 1:
            self.team_1['points'] += count_stich(cards, self.trumpf, last=last) * counting_factor[self.trumpf]
        else:
            self.team_2['points'] += count_stich(cards, self.trumpf, last=last) * counting_factor[self.trumpf]

    def get_player_number(self, player):
        for key, value in self.players.items():
            if player is value:
                return key


def get_player_key(start_key):
    for i in range(3):
        yield (i + start_key) % 4 + 1



import argparse
import logging
import sys

from pyschieber.player.cli_player import CliPlayer
from pyschieber.player.random_player import RandomPlayer
from pyschieber.tournament import Tournament


def start_tournament(points):
    tournament = Tournament(point_limit=points)
    cli_player = CliPlayer(name='CliPlayer')
    tournament.register_player(cli_player, 1)
    [tournament.register_player(RandomPlayer(name=str(i)), i) for i in range(2, 5)]
    tournament.play_game()


def set_logging():
    root = logging.getLogger()
    root.setLevel(logging.INFO)
    stream_handler = logging.StreamHandler(sys.stdout)
    stream_handler.setLevel(logging.INFO)
    formatter = logging.Formatter('%(message)s')
    stream_handler.setFormatter(formatter)
    root.addHandler(stream_handler)


if __name__ == "__main__":
    # logging.basicConfig(stream=sys.stdout, level=logging.INFO)
    set_logging()
    parser = argparse.ArgumentParser(description='CLI pyschieber', )
    parser.add_argument('-p', '--points', dest='points', type=int, help='Tournament points')
    parser.set_defaults(points=1500)
    args = parser.parse_args()
    start_tournament(args.points)



import logging

from pyschieber.game import Game
from pyschieber.player.base_player import BasePlayer

logger = logging.getLogger(__name__)


class Tournament:
    def __init__(self, point_limit=1500):
        self.point_limit = point_limit
        self.players = {}
        self.team_1 = dict(points=0, number=1)
        self.team_2 = dict(points=0, number=2)
        self.round = 1

    def start(self):
        self.check_players()
        self.init_teams()
        logger.info('Tournament starts, the goal are {} points.'.format(self.point_limit))

    def check_players(self):
        player_numbers = []
        for key, player in self.players.items():
            assert isinstance(player, BasePlayer)
            player_numbers.append(key)
        assert {1, 2, 3, 4} == set(player_numbers)

    def register_player(self, player, number):
        assert number in {1, 2, 3, 4}
        self.players[number] = player

    def init_teams(self):
        for key in self.players:
            if key % 2 == 1:
                self.team_1[str(key)] = self.players[key]
            else:
                self.team_2[str(key)] = self.players[key]

    def play_game(self):
        end = False
        while not end:
            game = Game(teams={1: self.team_1, 2: self.team_2}, point_limit=self.point_limit, players=self.players)
            logger.info('-' * 200)
            logger.info('Round {} starts.'.format(self.round))
            logger.info('-' * 200)
            end = game.start()
            logger.info('Round {} is over.'.format(self.round))
            logger.info('Points: Team 1: {0} , Team 2: {1}. \n'.format(self.team_1['points'], self.team_2['points']))
            self.round += 1
